import { Post } from '../models/Post';
import { EStatuses } from '../enums/Estatuses';
import IPost from '../interfaces/IPost';
import IResponse from '../interfaces/IResponse';
import IPostDto from '../interfaces/IPostDto';
import mongoose, { Mongoose } from 'mongoose'
import IDataBase from '../interfaces/IDataBase';

	export class Mongo implements IDataBase {
		private client: Mongoose | null = null
	 
		public close = async(): Promise<void> => {
			 if (!this.client) return
			 await this.client.disconnect()
		}
	 
		public async init(): Promise<void> {
		   this.client = await mongoose.connect(process.env.MONGO_CLIENT_URL || 'mongodb://localhost/myStore')
		   console.log('Mongo mongoose is connected')
		}
	 
		public getPosts= async(): Promise<IResponse<IPost[] | undefined>>=> {
		try{
			const data = await Post.find()
			const response: IResponse<IPost[]> = {
				status: EStatuses.OK,
				result: data as any,
				message: 'Posts found'
			}
			return response
		} catch (err: unknown) {
			const error = err as Error
			const response: IResponse<undefined> = {
				status: EStatuses.NOT_OK,
				result: undefined,
				message: error.message
			}
			return response
		}
	}
	public getPostById= async(id: string): Promise<IResponse<IPost | undefined>>=> {
		try{
			const data = await Post.findById(id)
			const response: IResponse<IPost> = {
				status: EStatuses.OK,
				result: data as any,
				message: 'Post found'
			}
			return response
		} catch (err: unknown) {
			const error = err as Error
			const response: IResponse<undefined> = {
				status: EStatuses.NOT_OK,
				result: undefined,
				message: error.message
			}
			return response
		}
	}
	public addPost = async(postDto: IPostDto): Promise<IResponse<IPost | undefined>>=> {
		try{
			const post = new Post(postDto)
			const data = await post.save()
			const response: IResponse<IPost> = {
				status: EStatuses.OK,
				result: data ,
				message: 'Post added'
			}
			return response
		} catch (err: unknown) {
			const error = err as Error
			const response: IResponse<undefined> = {
				status: EStatuses.NOT_OK,
				result: undefined,
				message: error.message
			}
			return response
		}
	}
	public deletePostById = async(id: string): Promise<IResponse<IPost | undefined>>=> {
		try{
			const data = await Post.findOneAndDelete({_id:id})
			const response: IResponse<IPost> = {
				status: EStatuses.OK,
				result: data as any,
				message: 'Post deleted'
			}
			return response
		} catch (err: unknown) {
			const error = err as Error
			const response: IResponse<undefined> = {
				status: EStatuses.NOT_OK,
				result: undefined,
				message: error.message
			}
			return response
		}
	}
	
};
export const mongo = new Mongo()

